////////////////////////////////////////////////////////////////////////////////
//  TXiCamVVRK1   -  
// 
////////////////////////////////////////////////////////////////////////////////
#ifndef XiCamVVRK1H
#define XiCamVVRK1H

#include <ScktComp.hpp>
#include "AdPort.hpp"


namespace Xi {
//--------- namespace Xi { -----------------------------------------------------



//------------------------------------------------------------------------------
//      Xi
//------------------------------------------------------------------------------
#ifndef XiFundamentalTypes
#define XiFundamentalTypes
typedef unsigned char UBYTE;     // .  
typedef unsigned short UWORD;    // 16-  ()
typedef unsigned int UDWORD;    // 32-  (.)
typedef unsigned int ERRCODE;    // 32-  (.)
#endif // #ifndef XiFundamentalTypes

//------------------------------------------------------------------------------
//  
//------------------------------------------------------------------------------
class TXiCamVVRK1;

////////////////////////////////////////////////////////////////////////////////
//  TXiCamVVRK1Connection    
////////////////////////////////////////////////////////////////////////////////
//------------------------------------------------------------------------------
// v.1.0.1.0 / 05.10.06    --  
//------------------------------------------------------------------------------
class TXiCamVVRK1Connection : public TObject {
   friend class TXiCamVVRK1;   //      
public:
  //------    --------------------------------------------
  //   ()
   enum TRWError
   {
      rweNone = 0x0,            //    
      rweRead = 0x1,            //   
      rweWrite = 0x2,           //   
      rweTimeOut = 0x4,         //  -   
   }; // enum TCommand

protected:
  //------  ----------------------------------------------------------------
   bool FActive;
   TXiCamVVRK1* FCamera;
   virtual void __fastcall SetActive(bool value) = 0;
   virtual bool __fastcall GetActive() = 0;
   virtual __int64 __fastcall SaveToStream(TStream* tStr, __int64 Count) = 0;   //   
   void InitCam();               //    - 

public:
   __fastcall TXiCamVVRK1Connection();
   virtual __fastcall ~TXiCamVVRK1Connection();
   virtual ERRCODE __fastcall Set_n_Get(void* OutBuf, UDWORD OutCnt,
      void* InBuf, UDWORD InCnt) = 0;
   __property bool Active = { read = GetActive, write = SetActive };
   __property TXiCamVVRK1* Camera = { read = FCamera /*, write = SetCamera*/ };

}; // class TXiCamVVRK1Connection


////////////////////////////////////////////////////////////////////////////////
//  TXiCamVVRK1_CnctnCOM --      COM-
////////////////////////////////////////////////////////////////////////////////
class TXiCamVVRK1_CnctnCOM : public TXiCamVVRK1Connection {
private:
  //------   -------------------------------------------------------
   TApdCustomComPort* F_ApdComPort;    // -,   COM-
   void __fastcall SetApdComPort(TApdCustomComPort* AdpComPort);
   void __fastcall SetActive(bool value);
   bool __fastcall GetActive();

   bool GettingShotData;

   void __fastcall ApdComPortDataTrig(TObject* CP, UWORD TriggerHandle);
   UWORD FShotEndTrig;           //    --  
   UWORD FShotGettingTimeout;    //    --   


public:
  //------    ------------------------------------------
   __fastcall TXiCamVVRK1_CnctnCOM(TApdCustomComPort* tApdComPort);
   virtual __fastcall ~TXiCamVVRK1_CnctnCOM();  // . 

  //    / ( )
   virtual ERRCODE __fastcall Set_n_Get(void* OutBuf, UDWORD OutCnt,
      void* InBuf, UDWORD InCnt);
  //      ,    ()
   virtual __int64 __fastcall SaveToStream(TStream* tStr, __int64 Count);

   __property TApdCustomComPort* AdpComPort = { read = F_ApdComPort,
      write = SetApdComPort };

}; // class TXiCamVVRK1_CnctnCOM : public TXiCamVVRK1Connection



////////////////////////////////////////////////////////////////////////////////
//  TXiCamVVRK1                                                      //
////////////////////////////////////////////////////////////////////////////////
class TXiCamVVRK1 : public TObject {
   friend class TXiCamVVRK1Connection;   //    
public:

  //------    --------------------------------------------

  //  --  
   typedef void __fastcall (__closure *TCaptureEvent)
      (TObject* Sender,  __int64 BytesWrote);

  // 
   enum TCommand
   {
      cmdGetState = 0x00,           //   
      cmdSetState = 0x10,           //   

      cmdGetUART = 0x01,            //   UART
      cmdSetUART = 0x11,            //   UART      

      cmdGetSPICtrl = 0x02,         //  \  
      cmdSetSPICtrl = 0x12,         //          /        SPI
      cmdGetBrigns = 0x03,          //  \  
      cmdSetBrigns = 0x13,          //  /      

      cmdGetLevelR = 0x04,          //  \  
      cmdSetLevelR = 0x14,          //  /      

      cmdGetLevelG = 0x05,          //  \  
      cmdSetLevelG = 0x15,          //  /      

      cmdGetLevelB = 0x06,          //  \  
      cmdSetLevelB = 0x16,          //  /      

      cmdGetSizeH = 0x07,           //  \  
      cmdSetSizeH = 0x37,           //  /  

      cmdGetSizeV = 0x08,           //  \  
      cmdSetSizeV = 0x38,           //  /  

      cmdTestModeOn = 0xA0,         //    
      cmdTestModeOff = 0xA1,        // -"-   

      cmdCapture = 0x40,            //  
      cmdLoadSnap = 0x80,           //  

      cmdResetCam = 0xE0,           //    

   }; // enum TCommand

  // 
   enum TRegister
   {
      rgnState = 0,                    //  
      rgnSPICtrl,                      // -/-  SPI
      rgnBrigns,                       // -/-  
      rgnLevelR,                       // -/-   
      rgnLevelG,
      rgnLevelB,
      rgnSizeH,                        //   \   
      rgnSizeV,                        //     /   
      rgnUART,                         //  UART

      rgn_high_value                   //     
   }; // enum TCommand

  //   SPI
   enum TSPIctrls
   {
      spic_0e01 = 0,          // _|_-  // . 0,  .
      spic_0e10,              // _|-_  // . 0,  .
      spic_1e10,              // -|-_  // . 1,  .
      spic_1e01               // -|_-  // . 1,  .

   }; // enum TSPIctrls

  // ,    ,   
  // /   
   struct TRegsCmds {
      //static const int MaxReg = 0;
      static const UBYTE GetWriteCmd(TRegister tReg)
      {
         if(tReg >= rgn_high_value)
            throw Exception(" ");

        //    
         static const UBYTE WriteRegCmd[] = { cmdSetState,
                                              cmdSetSPICtrl,
                                              cmdSetBrigns,
                                              cmdSetLevelR,
                                              cmdSetLevelG,
                                              cmdSetLevelB,
                                              cmdSetSizeH,
                                              cmdSetSizeV,
                                              cmdSetUART,
                                            };
         return WriteRegCmd[tReg];
      }

      static const UBYTE GetReadCmd(TRegister tReg)
      {
         if(tReg >= rgn_high_value)
            throw Exception(" ");
        //    
         static const UBYTE ReadRegCmd[] = { cmdGetState,
                                             cmdGetSPICtrl,
                                             cmdGetBrigns,
                                             cmdGetLevelR,
                                             cmdGetLevelG,
                                             cmdGetLevelB,
                                             cmdGetSizeH,
                                             cmdGetSizeV,
                                             cmdGetUART
                                           };
         return ReadRegCmd[tReg];
      }
   }; // struct TRegsCmds

private:
  //------  ----------------------------------------------------------------

  //     
   struct TBitsStateReg {
      UBYTE AspectRatio : 3;  // 3 .  -- . 
      UBYTE Coloured    : 1;  // 3  -- 
   }; // struct TStateReg

  // ,          
   union TStateReg {
      TBitsStateReg Bits;     // ,   
      UBYTE         Byte;     // ,   
   };



  //------  -----------------------------------------------------------
   static const int FSizeDelay = 60;   // 60     


  //------  ----------------------------------------------------------------
   TXiCamVVRK1Connection* FConnection;
   TClientSocket* FTestSocket;
   TStateReg FStateReg;                //   --  
   TSPIctrls FSPICtrl;
   UBYTE FBrightness;                  // 
   UBYTE FLevelR;                      //  \/
   UBYTE FLevelG;
   UBYTE FLevelB;
   UWORD FSizeH;                       //   .  
   UWORD FSizeV;
   bool FTestMode;
   bool FFrameMade;                    //   
   UBYTE FUART;
   TNotifyEvent FOnResponseWait;    //  
   TCaptureEvent FOnCapture;        //  
   void __fastcall SetColoured(bool value);
   bool __fastcall GetColoured() const;
   void __fastcall InitCam();
   UBYTE __fastcall RegisterExch(TRegister tReg, UBYTE tData = 0);



   void __fastcall SetRegister(TRegister tReg, UBYTE value);
   UBYTE __fastcall GetRegister(TRegister tReg);

   void __fastcall SetRegister2(TRegister tReg, UWORD value);
   UWORD __fastcall GetRegister2(TRegister tReg);

   DWORD FSleepTime; //  

   void __fastcall SetAspectRatio(unsigned char value);
   unsigned char __fastcall GetAspectRatio() const;
   void __fastcall SetSPICtrl(TSPIctrls value);
   void __fastcall SetBrightness(UBYTE value);
   void __fastcall SetLevelR(UBYTE value);
   void __fastcall SetLevelG(UBYTE value);
   void __fastcall SetLevelB(UBYTE value);
   void __fastcall SetSizeH(UWORD value);
   void __fastcall SetSizeV(UWORD value);
   void __fastcall SetTestMode(bool value);
   void __fastcall SetConnection(TXiCamVVRK1Connection* value);
   void __fastcall SetUART(UBYTE value);
   void __fastcall SetSleepTime(DWORD value);
   void __fastcall DoResponseWait(DWORD tTime);// const;


public:
   __fastcall TXiCamVVRK1();      //   
   __fastcall ~TXiCamVVRK1();
   void __fastcall ExCommand(TCommand);
   void __fastcall ResetCam();
   void __fastcall LoadCamState();
   void __fastcall Capture(TStream* tStr);
   void __fastcall LoadShotToFile();      // 

   __property TXiCamVVRK1Connection* Connection = { read = FConnection,
      write = SetConnection };
   __property bool Coloured = { read = GetColoured, write = SetColoured };
   __property UBYTE Registers[TRegister tReg] = { read = GetRegister,
      write = SetRegister};
   __property unsigned char AspectRatio = { read = GetAspectRatio, write = SetAspectRatio };
   __property TSPIctrls SPICtrl = { read = FSPICtrl, write = SetSPICtrl };
   __property UBYTE Brightness = { read = FBrightness, write = SetBrightness };
   __property UBYTE LevelR = { read = FLevelR, write = SetLevelR };
   __property UBYTE LevelG = { read = FLevelG, write = SetLevelG };
   __property UBYTE LevelB = { read = FLevelB, write = SetLevelB };
   __property UWORD SizeH = { read = FSizeH, write = SetSizeH };
   __property UWORD SizeV = { read = FSizeV, write = SetSizeV };
   __property bool TestMode = { read = FTestMode, write = SetTestMode };
   __property UBYTE UART = { read = FUART, write = SetUART };
   __property DWORD SleepTime = { read = FSleepTime, write = SetSleepTime };
   __property TNotifyEvent OnResponseWait =
      { read = FOnResponseWait, write = FOnResponseWait };
   __property TCaptureEvent OnCapture =
      { read = FOnCapture, write = FOnCapture };

}; // class TXiCamVVRK1 {



//--------- } namespace Xi -----------------------------------------------------
}; // namespace Xi {

//---------------------------------------------------------------------------
#endif
